#!/bin/bash

# please start me with    taskset -c 0 ./encode.sh      !

INFILE=st800x336.mkv
OUTBASE=st800x336
FPS=24
BITRATES="250 350 500 700 1000 1400 2000"

TIME="/usr/bin/time -o encode_times.txt --append"

DECODE="mplayer -quiet -vo null -nosound -benchmark"

$TIME -f"%e [R] $INFILE" $DECODE $INFILE

CMD="ffmpeg2theora --inputfps $FPS --optimize --two-pass"
for br in $BITRATES ; do
  OUTFILE=${OUTBASE}_theora_${br}kbps.ogv
  $TIME -f"%e [1] $OUTFILE" $CMD --first-pass  pass1.tmp -V $br -o /dev/null $INFILE
  $TIME -f"%e [2] $OUTFILE" $CMD --second-pass pass1.tmp -V $br -o $OUTFILE $INFILE
  $TIME -f"%e [D] $OUTFILE" $DECODE -vc theora $OUTFILE
  rm -f pass1.tmp
done

CMD="x264 --fps $FPS --threads 1 --slow-firstpass --bframes 3 --b-adapt 2 --b-pyramid strict --ref 4 --stats pass1.tmp --partitions all --direct auto --weightp 2 --me umh --subme 10 --trellis 2"
for br in $BITRATES ; do
  for profile in baseline main high ; do
    OUTFILE=${OUTBASE}_x264_${profile}_${br}kbps.264
    $TIME -f"%e [1] $OUTFILE" $CMD --bitrate $br --pass 1 --profile $profile -o /dev/null $INFILE
    $TIME -f"%e [2] $OUTFILE" $CMD --bitrate $br --pass 2 --profile $profile -o $OUTFILE  $INFILE
    $TIME -f"%e [D] $OUTFILE" $DECODE $OUTFILE
    rm -f pass1.tmp pass1.tmp.mbtree
  done
done

CMD="mencoder -fps $FPS -of rawvideo -ovc xvid"
OPTS="me_quality=6:qpel:trellis:nogmc:chroma_me:hq_ac:vhq=4:lumi_mask:max_bframes=2"
for br in $BITRATES ; do
  OUTFILE=${OUTBASE}_xvid_${br}kbps.m4v
  $TIME -f"%e [1] $OUTFILE" $CMD -xvidencopts $OPTS:bitrate=$br:pass=1 -o $OUTFILE $INFILE
  $TIME -f"%e [2] $OUTFILE" $CMD -xvidencopts $OPTS:bitrate=$br:pass=2 -o $OUTFILE $INFILE
  $TIME -f"%e [D] $OUTFILE" $DECODE $OUTFILE
  rm -f divx2pass.log
done

CMD="mencoder -fps $FPS -of rawvideo -ovc lavc"
OPTS="vcodec=mpeg2video:vme=4:mbd=2:keyint=18:lumi_mask=0.1:trell:cbp:mv0:subq=8:qns=3"
for br in $BITRATES ; do
  [ $br -lt 350 ] && continue
  OUTFILE=${OUTBASE}_mpeg2_${br}kbps.m2v
  $TIME -f"%e [1] $OUTFILE" $CMD -lavcopts $OPTS:vbitrate=$br:vpass=1 -o $OUTFILE $INFILE
  $TIME -f"%e [2] $OUTFILE" $CMD -lavcopts $OPTS:vbitrate=$br:vpass=2 -o $OUTFILE $INFILE
  $TIME -f"%e [D] $OUTFILE" $DECODE -vc ffmpeg2 $OUTFILE
  rm -f divx2pass.log
done

CMD="ffmpeg2dirac --inputfps $FPS --multi-quants --combined-me --mv-prec 1/8 --numL1 30 --sepL1 2"
for br in $BITRATES ; do
  OUTFILE=${OUTBASE}_dirac_${br}kbps.ogv
  $TIME -f"%e [1] $OUTFILE" $CMD -V $br -o $OUTFILE $INFILE
  $TIME -f"%e [D] $OUTFILE" $DECODE $OUTFILE
done
