#!/usr/bin/env python
import sys, os, re, stat, subprocess, tempfile
from eval import decode_to_y4m

ENCODER = [ "mplayer", "-really-quiet", "-nosound", "-benchmark", "-demuxer", "y4m", "-vo", "png:z=9" ]
OUTPUT_NAMES = "%08d.png"


re_width = re.compile(r' W(\d+)')
re_height = re.compile(r' H(\d+)')


def copy_y4m_frames(infile, outfile, frames):
    infile = open(infile, "rb")
    outfile = open(outfile, "wb")
    header = infile.readline()
    width = int(re_width.search(header).group(1))
    height = int(re_height.search(header).group(1))
    framesize = width * height * 3 / 2
    outfile.write(header)
    frameno = 0
    outcount = 0
    while True:
        frameno += 1
        sys.stderr.write("\r%d " % frameno)
        sys.stderr.flush()
        header = infile.readline()
        if not header.startswith("FRAME"):
            raise RuntimeError, "invalid YUV4MPEG frame"
        data = infile.read(framesize)
        if len(data) < framesize:
            break
        if frameno in frames:
            sys.stderr.write("-> output\n")
            sys.stderr.flush()
            outfile.write(header)
            outfile.write(data)
            outcount += 1
            if frameno >= max(frames):
                break
    sys.stderr.write("\r%d frames processed, %d frames output.\n" % (frameno, outcount))
    infile.close()
    outfile.close()
    return outcount


def extract_frames_from_stream(stream, frames):
    tmpdir = tempfile.mkdtemp()
    dec_fifo = os.path.join(tmpdir, 'dec.y4m')
    enc_fifo = os.path.join(tmpdir, 'enc.y4m')
    os.mkfifo(dec_fifo)
    os.mkfifo(enc_fifo)
    try:
        dec_prog = decode_to_y4m(stream, dec_fifo)
        enc_prog = subprocess.Popen(ENCODER + [enc_fifo], stdin=subprocess.PIPE)
        outcount = copy_y4m_frames(dec_fifo, enc_fifo, frames)
        dec_prog.wait()
        enc_prog.wait()
    finally:
        os.unlink(dec_fifo)
        os.unlink(enc_fifo)
        os.rmdir(tmpdir)

    base = os.path.splitext(stream)[0]
    ext = os.path.splitext(OUTPUT_NAMES)[-1]
    for i in xrange(outcount):
        os.rename(OUTPUT_NAMES % (i+1), "%s.frm%06d%s" % (base, frames[i], ext))


if __name__ == "__main__":
    if len(sys.argv) < 3:
        print "Usage:", os.path.basename(sys.argv[0]), "<FRAME1,[FRAME2,...]> <INPUT1> [<INPUT2> ...]"
        sys.exit(2)

    frames = map(int, sys.argv[1].split(','))
    frames.sort()
    
    for stream in sys.argv[2:]:
        if not os.path.isfile(stream):
            print "stream %s does not exist, skipping." % stream
            continue
        print "extracting frames %r from stream %s ..." % (frames, stream)
        extract_frames_from_stream(stream, frames)
